require "spec_helper"

shared_examples_for "a wiki page request" do
  # requires: obj, page
  it "should load the requested wiki page MyPage" do
    @obj.should_receive(:find_or_build_page).with("MyPage").and_return(my_page)
    do_request
  end

  it "should assign the wiki page for the view" do
    do_request
    assigns[:page].should == my_page
  end
end

shared_examples_for "a wiki start page request" do
  before do
    @obj.stub(:find_or_create_start_page).and_return(start_page)
  end

  it "should load the wiki page StartPage" do
    @obj.should_receive(:find_or_create_start_page).and_return(start_page)
    do_request
  end

  it "should assign the wiki page for the view" do
    do_request
    assigns[:page].should == start_page
  end

  it "should assign no page title for the view" do
    do_request
    assigns.should_not have_key(:page_title)
  end

  it "should display the index template" do
    do_request
    response.should render_template(:index)
  end
end

describe WikiController do
  let(:start_page) {mock_model(WikiPage, :start_page? => true)}
  let(:my_page) {mock_model(WikiPage, :start_page? => false, :name => "MyPage")}

  before do
    request.for_cms_object(@obj = mock_model(Wiki, :to_param => "123"))

    controller.stub(:ensure_object_is_active).and_return(true)
    controller.stub(:ensure_object_is_permitted).and_return(true)
    controller.stub(:enable_gui_to_update_its_menu_selection_in_preview_mode).and_return(true)
    controller.stub(:set_google_expire_header)
    @obj.stub(:find_or_build_page).and_return {|name|
      name == "StartPage" ? start_page : my_page
    }
  end

  describe "GET /123" do
    def do_request
      get 'index', :id => "123"
    end

    it_should_behave_like 'a wiki start page request'
  end


  describe "GET /123/list" do
    def do_request
      get 'list', :id => "123"
    end

    before do
      @pages = [mock("an array of wiki pages")]
      @obj.stub(:find_pages).and_return(@pages)
    end

    it "should assign the array of wiki pages for the view" do
      do_request
      assigns[:pages].should == @pages
    end

    it "should load all pages for the current wiki" do
      @obj.should_receive(:find_pages).and_return(@pages)
      do_request
    end

    it "should display the list template" do
      do_request
      response.should render_template(:list)
    end
  end

  describe "GET /123/show/MyPage" do
    def do_request
      get 'show', :id => "123", :name => 'MyPage'
    end

    it_should_behave_like 'a wiki page request'

    it "should render the show template" do
      do_request
      response.should render_template(:show)
    end

    it "should assign the page's name for the view" do
      do_request
      assigns[:page_title].should == "MyPage"
    end

  end


  describe "GET /123/show/StartPage" do
    def do_request
      get 'show', :id => "123", :name => 'StartPage'
    end

    it_should_behave_like 'a wiki start page request'
  end


  describe "GET /123/edit/MyPage" do
    def do_request
      get 'edit', :id => "123", :name => "MyPage"
    end

    it_should_behave_like 'a wiki page request'

    it "should display the edit template" do
      do_request
      response.should render_template(:edit)
    end

    it "should assign the page's name for the view" do
      do_request
      assigns[:page_title].should == "MyPage"
    end
  end


  describe "POST /123/edit/MyPage" do
    def do_request
      post 'edit', :id => "123", :name => "MyPage", :page => {:body => "My new content"}
    end

    before(:each) do
      my_page.stub(:update_attributes)
    end

    it_should_behave_like 'a wiki page request'

    it "should update the page MyPage with the new content" do
      my_page.should_receive(:update_attributes).with("body" => "My new content")
      do_request
    end

    it "should redirect to the show page" do
      do_request
      response.should redirect_to(:action => :show, :id => "123", :name => "MyPage")
    end
  end


  describe "delete /123/delete/MyPage" do
    def do_request
      delete :delete, :id => 123, :name => "MyPage"
    end

    before do
      @obj.stub(:delete_page)
    end

    it "should delete the page" do
      @obj.should_receive(:delete_page).with("MyPage")
      do_request
    end

    it "should redirect to the start page" do
      do_request
      response.should redirect_to(:action => :index, :id => "123")
    end
  end
end

describe WikiController do
  describe "#set_page_title(page)" do

    describe "for a start page" do
      let(:start_page) {mock_model(WikiPage, :start_page? => true)}

      it "should not assign a page title" do
        controller.__send__(:set_page_title, start_page)
        assigns[:page_title].should == nil
      end
    end

    describe "for a non-start page" do
      let(:my_page) {mock_model(WikiPage, :start_page? => false, :name => "MyPage")}

      it "should display the name of the wiki page" do
        controller.__send__(:set_page_title, my_page)
        assigns[:page_title].should == "MyPage"
      end
    end
  end
end
