require "spec_helper"

describe MyMenuHelper do
  before do
    helper.stub(:current_user).and_return(nil)
  end

  describe "(top_menu)" do
    it "should render a list of navigation links" do
      helper.top_menu.should have_tag('ul.level-1') do |ul|
        ul.should have_tag('li.first a[href="/wir-ueber-uns"]')
        ul.should have_tag('li a[href="/marken"]')
        ul.should have_tag('li a[href="/presse"]')
      end
    end
  end

  describe "(side_menu)" do
    before do
      assign(:obj, @obj = Obj.find_by_name('wirueberuns'))
      @obj.toclist.each do |o|
        helper.stub(:cms_path).with(o).and_return("/cms/#{o.name}")
      end
    end

    it "should render a list of navigation links" do
      helper.side_menu.should have_tag('ul.level-1') do |ul|
        ul.should have_tag('li.closed a[href="/cms/verantwortung"]')
        ul.should have_tag('li.closed a[href="/cms/faq"]')
        ul.should have_tag('li.closed a[href="/cms/geschichte"]')
        ul.should have_tag('li.closed a[href="/cms/stellenanzeigen"]')
        ul.should have_tag('li.closed a[href="/cms/impressum"]')
        ul.should have_tag('li.closed a[href="/cms/geschichte2"]')
      end
    end
  end

  describe '(menu)' do
    def mock_menu_obj(stubs = {})
      mock_model(Obj, {
        :sorted_toclist => [], :permitted_for_user? => true
      }.merge(stubs)).tap do |obj|
        if block_given?
          children = []
          yield children
          obj.stub(:sorted_toclist).and_return(children)
        end
      end
    end

    describe "menu, one level hierarchy" do

      before do
        @obj = mock_menu_obj do |c|
          c << mock_menu_obj(:display_title => "child1's title")
          c << @current = mock_menu_obj(:display_title => 'child2 & title')
          c << mock_menu_obj(:display_title => "child3's title")
        end
        helper.stub(:cms_path).and_return("/path")
        @current.stub(:ancestors).and_return [@obj]
      end

      let(:rendered_menu) { helper.menu(@obj, @current) }

      it "should render a one-level deep menu starting at the given object" do
        rendered_menu.should have_tag('ul', :count => 1)
        rendered_menu.should have_tag('ul > li', :count => 3)
      end

      it "should link all items but current" do
        rendered_menu.should have_tag('ul li a[href="/path"]', :count => 2)
        rendered_menu.should have_tag('ul') do |ul|
          ul.should have_tag('li:first-child a[href="/path"]')
          ul.should have_tag('li:last-child a[href="/path"]')
        end
      end

      it "should wrap the current object's item in a <span>" do
        rendered_menu.should have_tag('ul li:nth-child(2) span')
      end

      it "should render the quoted titles" do
        rendered_menu.should have_tag('ul') do |ul|
          ul.should have_tag('li', "child1's title", :inner_text => true)
          ul.to_s.should include('child2 &amp; title')
          ul.should have_tag('li', 'child2 & title', :inner_text => true)
          ul.should have_tag('li', "child3's title", :inner_text => true)
        end
      end

      it "should mark the list with the CSS class 'level-1'" do
        rendered_menu.should have_tag('ul.level-1')
      end

      it "should mark the first item with CSS class 'first'" do
        rendered_menu.should have_tag('li:first-child.first>a.first')
      end

      it "should mark the last item with CSS class 'last'" do
        rendered_menu.should have_tag('li:last-child.last>a.last')
      end

      it "should mark the current item with CSS class 'active'" do
        rendered_menu.should have_tag('li:nth-child(2).active>span.active')
      end

      it "should render a surrounding 'div'" do
        rendered_menu.should have_tag('div')
      end
    end


    describe "called without current_obj" do

      before do
        @obj = mock_menu_obj do |c|
          c << mock_menu_obj(:display_title => "child1's title")
          c << mock_menu_obj(:display_title => "child2's title")
          c << mock_menu_obj(:display_title => "child3's title")
        end
        helper.stub(:cms_path).and_return("/path")
      end

      let(:rendered_menu) { helper.menu(@obj, nil) }

      it "should link all items" do
        rendered_menu.should have_tag('ul li a[href="/path"]', :count => 3)
      end
    end


    describe "with option :menu_id given" do

      before do
        @obj = mock_menu_obj do |c|
          c << mock_menu_obj(:display_title => "child1's title")
        end
        helper.stub(:cms_path).and_return("/path")
      end

      let(:rendered_menu) {helper.menu(@obj, nil, :menu_id => 'my_nav')}

      it "should use this css ID for <div>" do
        rendered_menu.should have_tag('div#my_nav', :count => 1)
      end
    end


    describe "with option :hide_forbidden" do

      before do
        @obj = mock_menu_obj do |c|
          c << @current = mock_menu_obj(:display_title => "perm") do |cc|
            cc << mock_menu_obj(:display_title => "unperm child", :permitted_for_user? => false)
          end
          c << mock_menu_obj(:display_title => "unperm", :permitted_for_user? => false)
        end
        helper.stub(:cms_path).and_return("/path")
      end

      let(:rendered_menu) {helper.menu(@obj, @current, :hide_forbidden => true)}

      it "should not show children with unsatisfied live permissions" do
        rendered_menu.should have_tag('li', :count => 1)
      end
    end

    describe "without <li> to render" do

      before do
        @obj = mock_menu_obj
      end

      it "should render no <ul> at all" do
        helper.menu(@obj, nil).should_not have_tag('ul')
      end
    end


    describe "with two levels" do

      before do
        pub = nil
        @obj = mock_menu_obj do |c|
          c << mock_menu_obj(:display_title => "child1's title") do |cc1|
            cc1 << mock_menu_obj(:display_title => "child11's title")
          end
          c << mock_menu_obj(:display_title => "child2's title")
          c << pub = mock_menu_obj(:display_title => "child3's title") do |cc3|
            cc3 << mock_menu_obj(:display_title => "child31's title")
            cc3 << @current = mock_menu_obj(:display_title => "child32's title")
          end
        end
        helper.stub(:cms_path).and_return("/path")
        @current.stub(:ancestors).and_return [@obj, pub]
      end

      let(:rendered_menu) { helper.menu(@obj, @current) }

      it "should render a two-level deep menu starting at the given object" do
        rendered_menu.should have_tag('ul ul', :count => 1)
        rendered_menu.should have_tag('ul li:last-child ul')
      end

      it "should unfold the branch to the current object" do
        rendered_menu.should have_tag('ul.level-1>li', :count => 3)
        rendered_menu.should have_tag('li', :count => 5) # all(7) w/o obj, c11
        rendered_menu.should have_tag('li.closed', :count => 3)
        rendered_menu.should have_tag('li.open', :count => 2)
      end

      it "should mark the top-level list with the CSS class 'level-1'" do
        rendered_menu.should have_tag('ul.level-1')
      end

      it "should mark the second-level list with the CSS class 'level-2'" do
        rendered_menu.should have_tag('ul ul.level-2')
      end

      it "should mark the first items on each level with CSS class 'first'" do
        rendered_menu.should have_tag('li:first-child.first>a.first')
        rendered_menu.should have_tag('li:last-child li:first-child.first>a.first')
      end

      it "should mark the last items on each level with CSS class 'last'" do
        rendered_menu.should have_tag('li:last-child.last>a.last')
        rendered_menu.should have_tag('li:last-child li:last-child.last>span.last')
      end

      it "should mark the current item with CSS class 'active'" do
        rendered_menu.should have_tag('li:last-child li:last-child.active>span.active')
      end

      it "should mark the open items with CSS class 'open'" do
        rendered_menu.should have_tag('ul.level-1>li:last-child.open>a.open')
        rendered_menu.should have_tag('ul.level-1>li.open', :count => 1)
      end

      it "should mark the closed items with CSS class 'closed'" do
        rendered_menu.should have_tag('ul.level-1>li:first-child.closed>a.closed')
      end
    end


    describe "with 5 levels" do

      before do
        l1 = l2 = nil
        @obj = mock_menu_obj do |c1|
          c1 << l1 = mock_menu_obj(:display_title => "L1") do |c11|
            c11 << l2 = mock_menu_obj(:display_title => "L2") do |c111|
              c111 << @current = mock_menu_obj(:display_title => "L3") do |c1111|
                c1111 << mock_menu_obj(:display_title => "L4") do |c11111|
                  c11111 << mock_menu_obj(:display_title => "L5")
                end
              end
            end
          end
        end
        helper.stub(:cms_path).and_return("/path")
        @current.stub(:ancestors).and_return [@obj, l1, l2]
      end

      it "should not render deeper than current_obj's children" do
        helper.menu(@obj, @current).should have_tag('ul.level-4')
        helper.menu(@obj, @current).should_not have_tag('ul.level-5')
      end

      it "should render only one level with current_obj == nil" do
        helper.menu(@obj, nil).should have_tag('ul.level-1')
        helper.menu(@obj, nil).should_not have_tag('ul.level-2')
      end

      it "should not render deeper than max_depth" do
        helper.menu(@obj, @current, :max_depth => 2).should have_tag('ul.level-2')
        helper.menu(@obj, @current, :max_depth => 2).should_not have_tag('ul.level-3')
      end

      it "should not render deeper than current_obj's children even if max_depth is higher" do
        helper.menu(@obj, @current, :max_depth => 5).should have_tag('ul.level-4')
        helper.menu(@obj, @current, :max_depth => 5).should_not have_tag('ul.level-5')
      end

      it "should not limit the depth if max_depth is :unlimited" do
        helper.menu(@obj, @current, :max_depth => :unlimited).should have_tag('ul.level-4')
      end

      it "should use the current_time option to evaluate the toclist" do
        current_time = 1.year.from_now
        children = @obj.sorted_toclist
        @obj.should_receive(:sorted_toclist).with(current_time).and_return(children)
        helper.menu(@obj, nil, :current_time => current_time)
      end

      it "should mark the current item with CSS class 'active'" do
        helper.menu(@obj, @current).should have_tag('ul.level-1 ul.level-2 ul.level-3 li.active>span.active')
      end

      it "should mark the open items with CSS class 'open'" do
        helper.menu(@obj, @current).should have_tag('ul.level-1>li.open>a.open')
        helper.menu(@obj, @current).should have_tag('ul.level-2>li.open>a.open')
        helper.menu(@obj, @current).should have_tag('ul.level-3>li.open>span.open')
        helper.menu(@obj, @current).should_not have_tag('ul.level-4>li.open')
      end

      it "should not mark the open items with CSS class 'open'" +
      " if the children are not displayed due to max_depth" do
        helper.menu(@obj, @current, :max_depth => 2).should_not have_tag('ul.level-2>li.open')
      end

      it "should mark the closed items with CSS class 'closed'" do
        helper.menu(@obj, @current).should have_tag('ul.level-4>li.closed>a.closed')
      end

      it "should not mark the closed items with CSS class 'closed'" +
      " if the children are not displayed due to max_depth" do
        helper.menu(@obj, @current, :max_depth => 4).should_not have_tag('ul.level-4>li.closed')
      end

      it "should render the deepest displayable item on current path as link with CSS class 'active'" do
        helper.menu(@obj, @current, :max_depth => 2).should have_tag('ul.level-2>li.active>a.active')
      end

      it "should not give the CSS class 'active' to every item on the current path" do
        helper.menu(@obj, @current).should_not have_tag('ul.level-1>li.active')
        helper.menu(@obj, @current).should_not have_tag('ul.level-2>li.active')
      end
    end


    describe "tree with multiple branches and :unfold => :all" do

      before do
        @obj = mock_menu_obj do |c|
          c << mock_menu_obj(:display_title => "a") do |a|
            a << mock_menu_obj(:display_title => "aa1")
            a << mock_menu_obj(:display_title => "aa2")
          end
          c << mock_menu_obj(:display_title => "b") do |b|
            b << mock_menu_obj(:display_title => "bb1")
            b << mock_menu_obj(:display_title => "bb2")
            b << mock_menu_obj(:display_title => "bb3")
          end
        end
        helper.stub(:cms_path).and_return("/path")
      end

      let(:rendered_menu) { helper.menu(@obj, nil, :unfold => :all) }

      it "should render all branches" do
        rendered_menu.should have_tag("ul", :count => 3)
        rendered_menu.should have_tag("ul li ul li", :count => 5)
      end

      it "should mark the open items with CSS class 'open'" do
        rendered_menu.should have_tag('ul.level-1>li.open>a.open', :count => 2)
        rendered_menu.should have_tag('li.open>a.open', :count => 7)
      end
    end


    describe "called with a block" do

      before do
        @obj = mock_menu_obj do |c|
          c << mock_menu_obj(:display_title => "child1's title", :sort_value => "Z")
          c << mock_menu_obj(:display_title => "child2's title", :sort_value => "A")
          c << mock_menu_obj(:display_title => "child3's title", :sort_value => "B")
        end
        helper.stub(:cms_path).and_return("/path")
      end

      let(:rendered_menu) {
        helper.menu(@obj, nil) do |items, level|
          items.sort_by {|item| item.sort_value}
        end
      }

      it "should renders the items as they are returned by the block" do
        rendered_menu.should have_tag('ul') do |ul|
          ul.should have_tag("li:nth-child(1)", "child2's title", :inner_text => true)
          ul.should have_tag("li:nth-child(2)", "child3's title", :inner_text => true)
          ul.should have_tag("li:nth-child(3)", "child1's title", :inner_text => true)
        end
      end
    end
  end

end
